/**
 * Copyright Marcus Britanicus <marcusbritanicus@gmail.com>
 *
 * This file is a part of LibDesQ Widgets (https://gitlab.com/DesQ/libdesqui)
 * This library contains the UI classes and Plugin interfaces that are used
 * through out the DesQ Project.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * at your option, any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 **/

#pragma once

#include <QApplication>

#if defined(qApp)
#undef qApp
#endif

#define qApp    (static_cast<DesQ::Application *>(QCoreApplication::instance() ) )

class DesQIpcServer;
class QLockFile;
class QWidget;

namespace DesQ {
    class Application;
}

class DesQ::Application : public QApplication {
    Q_OBJECT;

    public:
        Application( const QString& id, int& argc, char **argv );
        ~Application();

        bool isRunning();
        QString id() const;

        DesQIpcServer *ipcServer();

        void setActivationWindow( QWidget *aw, bool activateOnMessage = true );
        QWidget * activationWindow() const;

    public Q_SLOTS:
        bool sendMessage( const QString& message );
        void activateWindow();
        void disconnect();

    Q_SIGNALS:
        void messageReceived( const QString& message );

    private:
        QLockFile *lockFile       = nullptr;
        DesQIpcServer *mIpcServer = nullptr;

        QWidget *actWin;

        QString mSocketName;
        QString mAppId;
};
